/* vi:set tabstop=2 shiftwidth=2 cindent: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 *
 * super.c: VxFS superblock and mount routines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: super.c,v 0.3 1999/12/14 19:04:35 mhi Exp root $
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <libvxfs.h>

int
vx_readsuper (struct vxfs *fs)
{
	struct vxfs_super *sb;

	sb = (struct vxfs_super *)vxread (fs, 1, 1);
	if (sb->magic != VXFS_MAGIC)
		{
			if (vxverbose)
				fprintf (stderr, "read_super: Invalid magic number (magic=%08x)\n",
								 sb->magic);
			return -1;
		}

	fs->oltext = sb->oltext[0];
	fs->oltsize = sb->oltsize;

	/*
	 * XXX: superblock checksum 
	 */

	if (sb->version != 2 && sb->version != 4 && vxverbose) {
		fprintf(stderr,"vxreadsuper: Warning: You have an unsupported VxFS version.\n");
		fprintf(stderr,"Please contact author.\n");
	}

	if (vxdebug)
		{
			printf ("VxFS version = %u\n", sb->version);
			printf ("Volume label = %s\n", sb->fname);
			printf ("Pack label = %s\n", sb->fpack);
		}
	free (sb);
	return 0;
}

struct vxfs *
vx_open (char *device, int subpart)
{
	struct vxfs *fs;

	fs = (struct vxfs *) malloc (sizeof (struct vxfs));
	if (!fs)
		return NULL;

	fs->fd = -1;
	fs->bsize = -1;
	fs->partoffset = -1;
	fs->iext = -1;
	fs->fshino = -1;

	if (vxdebug)
		printf ("Opening VxFS %s..\n", device);

	if (vxopendev (fs, device) < 0)
		{
			free (fs);
			if (vxverbose)
				fprintf (stderr, "vx_open: vxopendev failed\n");
			return NULL;
		}

	if (vxsetpart (fs, subpart) < 0)
		{
			free (fs);
			if (vxverbose)
				fprintf (stderr, "vx_open: vxsetpart failed\n");
			return NULL;
		}

	if (vxdebug)
		printf ("Reading filesystem structures...\n");

	if (vx_readsuper (fs) < 0)
		{
			free (fs);
			return NULL;
		}

	if (vx_readolt (fs) < 0)
		{
			free (fs);
			return NULL;
		}

	if (vx_readfshead (fs) < 0)
		{
			free (fs);
			return NULL;
		}

	if (vxdebug)
		printf ("VxFS initialized\n");

	return fs;
}
